;**********************
;**** Clear Screen ****
;********************************************
;** Duh....I think this clears the screen	**
;********************************************

ClearScreen:
	move.l	#doorsos::main_lcd,a0	
	move.w	#959,d0				

\EraseLoop
	clr.l		(a0)+				
	dbf.w		d0,\EraseLoop		
	rts	

;***************
;**** Block ****
;*********************************
;** Erases the cursor from its	**
;** old spot, and puts it in 	**
;** its new one.			**
;*********************************

Block:
	move.w	OldX,d0				
	move.w	OldY,d1			
	move.w	#7,d2			
	move.w	#7,d3				
	move.w	#0,d4
	jsr		graphlib::fill
	move.w	CurX,d0				
	move.w	CurY,d1			
	move.w	#7,d2			
	move.w	#7,d3				
	move.w	#0,d4
	jsr		graphlib::fill
	rts
	
;******************
;*** Half Block ***
;**************************************
;** Temporarily removes cursor block	**
;** or puts it back when it's called 	**
;** twice.						**
;**************************************

HalfBlock:

	move.w	CurX,d0		;<-- move CurX into the argument for X-Coordinate 
	move.w	CurY,d1		;<-- move CurY into the argument for Y-Coordinate
	move.w	#7,d2			;<-- move 7 into the width argument 
	move.w	#7,d3			;<-- move 7 into the height argument 
	move.w	#0,d4			;<-- move 0 into the copy method argument
	jsr		graphlib::fill	;<-- getrid of all these arguments by calling
						;      the function
	rts

;********************
;**** Sprite Put ****
;************************************************************
;** d0 = x										**
;** d1 = y										**
;** d2 = copy method:		<-- Not yet implemented---- 	**
;**			0 : Invert video			     V	**
;**			1 : Black (1 = pixel on in sprite data)	**
;**			2 : White (1 = pixel off in sprite data)	**
;** a1 = address of sprite data						**
;************************************************************

SpritePut:
	divu.w	#8,d0
	move.l	d0,d3
	and.l		#$FFFF0000,d3
	and.l		#$0000FFFF,d0
	ext.l		d0	
	swap		d3
	
	move.l	#doorsos::main_lcd,a0
	mulu.w	#30,d1
	move.w	d0,d3
	move.w	#30,d4
	sub.w		d3,d4
	add.w		d1,a0

	move.w	#7,d6
\SprLoop
	add.w		d0,a0
	move.b	(a1)+,(a0)
	add.w		d4,a0
	dbf.w		d6,\SprLoop

	rts

;**************************
;**** Block Number Put ****
;*******************************************
;** Puts the correct number tile on the   **
;** screen at the given location          **
;*******************************************


BlkNumberPut:
	move.w	CurX,d0				
	move.w	CurY,d1			
	move.w	#7,d2			
	move.w	#7,d3				
	move.w	#0,d4
	jsr		graphlib::fill
	move.w	BombCounter,d0
	mulu.w	#8,d0
	move.l	#BlkNums,a1
	add.w		d0,a1
	move.w	CurX,d0
	move.w	CurY,d1
	bsr		SpritePut
	move.w	CurX,d0				
	move.w	CurY,d1			
	move.w	#7,d2			
	move.w	#7,d3				
	move.w	#0,d4
	jsr		graphlib::fill
	rts

;***********************
;**** Display Board ****
;*******************************
;** This routine sets up the  **
;** game board. It came out   **
;** of the old Read Level     **
;** routine. Now it just puts **
;** blank blocks on the       **
;** screen and couldn't care  **
;** less about actual level   **
;** data.                     **
;*******************************

DisplayBoard:
	move.w	#9,d4				;OUTER LOOP ----------
							;				^
\LvlLineLoop					;				|
	move.w	#15,d5			;  INNER LOOP		|
							;	^			|
\LvlColumnLoop					;	|			|
	move.l	#BlkNew,a1			;	|			|
	move.w	d5,d0				;	|			|
	mulu.w	#8,d0				;	|			|
	add.w		#16,d0			;	|			|
	move.w	d4,d1				;	|			|
	mulu.w	#8,d1				;	|			|
	add.w		#16,d1			;	|			|
	move.l	d5,d5b			;	|			|
	move.l	d4,d4b			;	|			|
	jsr		SpritePut			;	|			|
	move.l	d5b,d5			;	|			|
	move.l	d4b,d4			;	|			|
	dbf.w		d5,\LvlColumnLoop	;  INNER Loop end	|
							;			    	V
	dbf.w		d4,\LvlLineLoop		;OUTER Loop end-------


	PutNum2	#0,#25,Flags			;<-- Displays the number of flags
	move.w	#0,d0					;<-- We need to display the flag icon, 
	move.w	#16,d1				;<-- BlkFlag, at (0,16) on the screen
	move.l	#BlkFlag,a1			;<-- moves the ADDRESS of BlkFlag into a1
	jsr		SpritePut				;<-- Puts the flag icon on the screen
	PutNum3	#0,#141,Seconds			;<-- Displays the number of seconds
	move.w	#133,d0				;<-- We need to display the timer icon, 
	move.w	#8,d1					;<-- BlkTimer, at (133,8) on the screen
	move.l	#BlkTimer,a1			;<-- moves the ADDRESS of BlkFlag into a1
	jsr		SpritePut				;<-- Puts the flag icon on the screen
	Square2	#16,#0,#101,#13			;<-- Title Box
	String	#18,#3,#16,#2,StrTitle	;<-- Display Title
	jsr		Block					;<-- Puts the cursor on the screen

	rts

;*************************
;**** Display Seconds ****
;*********************************
;** Updates the on-screen clock	**
;*********************************

DispSec:
	cmp.w		#999,Seconds
	bne		\DispSecContinue
	move.w	#0,Seconds
\DispSecContinue
	PutNum3	#141,#8,Seconds			;<-- A macro that displays a three digit
								;	number. In this case, the number of
								;	seconds on the clock.
	add.w		#1,Seconds				;<-- Increase seconds on timer.
	rts

PutChar1:		
	
	move.w	NameOffset,d0
	mulu.w	#6,d0
	add.w		#20,d0
	move.w	#4,d1
	move.w	#6,d2
	move.w	#8,d3
	move.w	#1,d4
	jsr		graphlib::fill
	move.w	NameOffset,d0
	mulu.w	#6,d0
	add.w		#20,d0
	WriteStr	d0,#4,#0,Char1
	rts
			
