;**************************************************
;**** NOTE:                                    ****
;**** LvlData, FlagData, and WinData are all   ****
;**** parallel arrays (16x10 = 160 blocks)     ****
;**** that use the same addressing and data    ****
;**** system. 1 represents a bomb in level     ****
;**** data, a flag in FlagData, and a          ****
;**** tested/flagged block in Win data. 0      ****
;**** represents the absence of these items in ****
;**** the data. The arrays are 1 word wide and ****
;**** 10 rows in length = 10 words or 20 bytes ****
;**** each.                                    ****
;**************************************************






;********************
;**** Bomb Count ****
;**************************************
;** Counts to see how many bombs     **
;** are surrounding the block, and   **
;** to see if the block itself is a  **
;** bomb.                            **
;**                                  **
;** Note:                            **
;** -- The tests go clockwise        **
;** starting to the left of the      **
;** current selected block.	     **
;**						     **
;** -- (0,0) is the top left corner, **
;** and (15,9) is the bottom right   **
;** corner of the puzzle.            **
;**************************************

BombCount:
	move.w	#0,BombCounter
	move.l	#FlagData,a0
	move.w	BlkY,d0
	mulu.w	#2,d0
	add.w		d0,a0
	move.w	(a0),d1
	move.w	BlkX,d0
	btst.l	d0,d1
	beq		\BlkContinue    ;<-- If it's flagged, then we can't test it
	rts
\BlkContinue
	move.w	BlkX,BX		;<-- Save the current block location
	move.w	BlkY,BY		;<--   into BX and BY
	BlkTest	BlkX,BlkY		;<-- Is Current Block a bomb?
	bne		BlkIsBomb		;<-- If so branch to "Block is a bomb!!!"
	cmp.w		#0,BX			;<-- Is the cursor all the way to the left?
	beq		\BlkTestUp		;<-- If so brach to "Test Upwards"

\BlkTestLeft				
	sub.w		#1,BlkX		;<-- Move the test block one to the left
	BlkTest	BlkX,BlkY		;<-- Test it
	beq		\BlkTestUpLeft	;<-- If Block was a zero (aka. equal flag set)
						;      then don't count it as a bomb and skip
						;	  past the next instruction
	add.w		#1,BombCounter	;<-- Add one to the bomb counter

\BlkTestUpLeft
	cmp.w		#0,BY			;<-- Is cursor all the way at the top?
	beq		\BlkTestRight	;<-- If so branch to "Test to the Right"
	sub		#1,BlkY		;<-- BlkX and BlkY have been lowered by one
						;	  to get to the upper left of the 
						;	  selected block (BlkX was lowered in the 
						;	  last test)
	BlkTest	BlkX,BlkY		;<-- Is it a bomb?
	beq		\BlkTestUp		;<-- If not, skip the counter increment
	add.w		#1,BombCounter	;<-- If so, add one to the bomb counter
	
\BlkTestUp
	move.w	BX,BlkX		;<-- Restore the testing block position to
	move.w	BY,BlkY		;<--   the current cursor location
	cmp.w		#0,BY			;<-- Is the cursor at the top?
	beq		\BlkTestRight	;<-- If so, skip to "Test to the Right"
	sub.w		#1,BlkY		;<-- If not, then move the test up one
	BlkTest	BlkX,BlkY		;<-- Test it.
	beq		\BlkTestUpRight ;<-- If not a bomb, skip next line
	add.w		#1,BombCounter	;<-- If it is a bomb, pile on the counters

\BlkTestUpRight
	cmp.w		#15,BX		;<-- Is the cursor on the right edge?
	beq		\BlkTestDown	;<-- If so, skip to "Test downwards"
	add.w		#1,BlkX		;<-- Move to the right (We already moved up)
	BlkTest	BlkX,BlkY		;<-- Test it
	beq		\BlkTestRight	;<-- If not a bomb, skip counter increment
	add.w		#1,BombCounter	;<-- If so, chock another one up for
						;      the ol' bomb counter

\BlkTestRight
	move.w	BX,BlkX		;<-- Restore test block to current 
	move.w	BY,BlkY		;<--   cursor position
	cmp.w		#15,BX		;<-- Is cursor on the right hand edge?
	beq		\BlkTestDown	;<-- If so, skip to "Test downwards"
	add.w		#1,BlkX		;<-- Move to the right
	BlkTest	BlkX,BlkY		;<-- Test block.
	beq		\BlkTestDownRight	;<-- If not a bomb, JUUUUUUMP!!!
	add.w		#1,BombCounter	;<-- Boy, that bomb counter is getting fat.

\BlkTestDownRight
	cmp.w		#9,BY			;<-- Is cursor on the bottom edge?
	beq		\BlkDoneTest	;<-- If so, then we're all done!
	add.w		#1,BlkY		;<-- Move test block down (already moved it right)
	BlkTest	BlkX,BlkY		;<-- Test the block
	beq		\BlkTestDown	;<-- What! You're not a bomb! Go to 
						;      "Test Downwards" right this instant!!!
						;	  And don't give me any of that lip!
	add.w		#1,BombCounter	;<-- Oh, the humanity!! Look at all those bombs!
	
\BlkTestDown
	cmp.w		#9,BY			;<-- Is the cursor on the bottom edge?
	beq		\BlkDoneTest	;<-- If so, then print the number of bombs
						;<--   we collected by branching to "Done testing"
	move.w	BX,BlkX		;<-- If not, Restore the testing block
	move.w	BY,BlkY		;<--   to the current cursor position
	add.w		#1,BlkY		;<-- Move down.
	BlkTest	BlkX,BlkY		;<-- Is it a bomb? Well? Is it or not?!.
	beq		\BlkTestDownLeft;<-- It's not. Wheew. That was close!
						;      Now let's go to "Test down and to the left
	add.w		#1,BombCounter	;<-- IT IS A BOMB! RUUUN!!!!!! (and add one to
						;	  the counter.)
\BlkTestDownLeft			;<-- The FINAL test (oooh scary!)
	cmp.w		#0,BX			;<-- Are we on the left edge?
	beq		\BlkDoneTest	;<-- If so, then we're all done with the tests
						;	  (Don't worry, you'll walk fine in a couple
						;	  days. tee-hee-hee)
	sub.w		#1,BlkX		;<-- Move the testing block to the left
	BlkTest	BlkX,BlkY		;<-- Zit a bomb? (if so I'd hate to be
						;	  around when it pops.)
	beq		\BlkDoneTest	;<-- If not, then the tests are done
	add.w		#1,BombCounter	;<-- If so, then add one to the bomb counter

\BlkDoneTest				;<-- Well, that didn't take long. 
						;	  What's a couple 'a years anyway?
	move.w	BX,BlkX		;<-- Restore the current cursor position
	move.w	BY,BlkY		;<--   back into the approprieate variables
	move.l	#WinData,a0	;<-- Get address of win data, which is a matrix
						;	  the size of the level, but it's bits are
						;	  set when either a block is flagged or it	
						;	  is opened. We are opening a block, thus
						;	  we set the approprieate bit in WinData
	move.w	BlkY,d1		
	mulu.w	#2,d1			;<-- Each line is 2 bytes not 1
	add.w		d1,a0			
	move.w	(a0),d2		;<-- Get the correct line
	move.w	BlkX,d1		
	bset.l	d1,d2			;<-- Set it's bits
	move.w	d2,(a0)		;<-- And replace the line back where it belongs
	jsr 		BlkNumberPut	;<-- Time to print the results...
	jsr		CheckForWin	;<-- Did we win???
	rts					;<-- Return to "Keyboard test loop" from
						;	  whence we cameth

BlkIsBomb:
	jsr		HalfBlock		;<-- Get rid of cursor
	move.w	BX,d0			;<-- Move BX into d0 for manipulation
	mulu.w	#8,d0			;<-- Sprites are 8 bits wide
	add.w		#16,d0		;<-- The first collumn is 16 bits from the left
	move.w	BY,d1			;<-- Move BY into d1 for manipulatin
	mulu.w	#8,d1			;<-- Sprites are 8 bits High
	add.w		#16,d1		;<-- The first row is 16 from the top
	move.l	#BlkBomb,a1	;<-- Give the address of the tile we want
	jsr		SpritePut		;<-- Put the sprite on screen
	jsr		HalfBlock		;<-- Put the Cursor back
	move.b	#-1,WinFlag	;<-- Signal that the game is LOST
	rts

;**************
;**** Flag ****
;********************************************
;** Either puts down a flag, or picks a    **
;** flag up. If there is already a flag    **
;** in the block it picks it up, otherwise **
;** it puts one down. The routine also     **
;** checks to see if the block has been    **
;** opened or not.                         **
;********************************************

Flag:
	move.l	#FlagData,a0	;<-- Get the address of the flag data
	move.w	BlkY,d0		;<-- Move the block's row into d0
	mulu.w	#2,d0			;<-- Each row of game data is 2 bytes
	add.w		d0,a0			;<-- Add the offset of the block's row to 
						;	the address of the flag data 
	move.w	(a0),d1		;<-- Move 1 word of level data into d1 for manip.
	move.w	BlkX,d0		;<-- Each collumn of level data is only 1 bit wide
						;	so no manipulation is nessisary
	btst.l	d0,d1			;<-- Test the block of level data
	bne		\FlagIsSet		;<-- If the flag is set, jump to "Flag is set"

	tst.w		Flags			;<-- Otherwise, test to see if you have flags
						;    and if the block is tested or not
	bne		\CheckOpenBlk	
	rts					;<-- No Flags, so lets go home.

\CheckOpenBlk
	move.l	#WinData,a0	;<-- Before we can put down a flag we must
	move.w	BlkY,d0		;<-- first check to see if the block has 
	mulu.w	#2,d0			;<-- been tested.
	add.w		d0,a0			;<-- I use the same procedure every time
	move.w	(a0),d1		;<-- See above code for an explanation of how
	move.w	BlkX,d0		;<-- it works.
	btst		d0,d1
	beq		\PutDownFlag	;<-- It's untested, we're cool.
	rts					;<-- Uh-oh it's been tested. Let's get out of here!

\PutDownFlag
	move.l	#FlagData,a0	;<-- Update flag data
	move.w	BlkY,d0		;<-- Get the offset for the data we want.
	mulu.w	#2,d0			;<-- See the begining of "Flag" for an
	add.w		d0,a0			;<-- explanation of how it works
	move.w	(a0),d1		
	move.w	BlkX,d0
	bset.l	d0,d1
	move.w	d1,(a0)
	move.l	#WinData,a0	;<-- Update win data
	move.w	BlkY,d0
	mulu.w	#2,d0
	add.w		d0,a0
	move.w	(a0),d1
	move.w	BlkX,d0
	bset.l	d0,d1
	move.w	d1,(a0)
	sub.w		#1,Flags		;<-- We used a flag and now have one less
	PutNum2	#0,#25,Flags	;<-- Update the display
	jsr		HalfBlock		;<-- Get rid of cursor
	move.w	BlkX,d0		;<-- Show the flag tile.
	mulu.w	#8,d0			;<-- See above for how it works
	add.w		#16,d0		
	move.w	BlkY,d1		
	mulu.w	#8,d1			
	add.w		#16,d1		
	move.l	#BlkFlag,a1	
	jsr		SpritePut		;<-- Display a flag at the approprieate spot
	jsr		HalfBlock		;<-- Show cursor again
	jsr		CheckForWin	;<-- Did we win??
	rts					;<-- return to key loop

\FlagIsSet
	move.l	#FlagData,a0	;<-- Update flag data
	move.w	BlkY,d0		
	mulu.w	#2,d0			
	add.w		d0,a0			
	move.w	(a0),d1		
	move.w	BlkX,d0		
	bclr.l	d0,d1			
	move.w	d1,(a0)		
	move.l	#WinData,a0	;<-- Update win data
	move.w	BlkY,d0		
	mulu.w	#2,d0			
	add.w		d0,a0			
	move.w	(a0),d1		
	move.w	BlkX,d0		
	bclr.l	d0,d1			
	move.w	d1,(a0)		
	add.w		#1,Flags		
	PutNum2	#0,#25,Flags	;<-- Update the display
	jsr		HalfBlock		;<-- Get rid of cursor
	move.w	BlkX,d0		;<-- Show the new block tile
	mulu.w	#8,d0			
	add.w		#16,d0		
	move.w	BlkY,d1		
	mulu.w	#8,d1			
	add.w		#16,d1		
	move.l	#BlkNew,a1		
	jsr		SpritePut		;<-- Display a New block at the appropreate spot
	jsr		HalfBlock		;<-- Show cursor again
	rts					;<-- return to key loop
	
;***********************
;**** Check for Win ****
;*********************************************
;** Checks to see if all of the Win data is **
;** a one and not a zero. Every flag counts **
;** as a one. Every opened block counts as  **
;** a one. If you have either opened or     **
;** flagged every block, then you have won. **
;*********************************************

CheckForWin:
	move.l	#WinData,a0	;<-- Load up the address of the Win data
	move.w	#9,d7			;<-- We need to loop 10 times.

\CheckLoop
	move.w	(a0)+,d0		;<-- Get one row of data
	cmp.w		#$ffff,d0		;<-- Are all the blocks in this row either 
						;	tested or flagged? aka. all 1's no 0's
						;NOTE: The hexadecimal number $ffff equals
						;      the binary number %1111111111111111
	bne		\NoWin		;<-- If there are any 0's in the data, NO WIN.
	dbf.w		d7,\CheckLoop	;<-- Loops until d7 = -1, then it goes on
	jmp		YouWin		;<-- We made it through the loop!! WE WON!!

\NoWin
	rts					;<-- We return defeated. Damn.

YouWin:
	move.b	#1,WinFlag		;<-- Set the "We kicked ass" flag.
	rts					;<-- And return victorious!

;********************
;**** Make Level ****
;*************************************************
;** It first resets all level data,flag data,	**
;** and Win data. Then it adds the approprieate	**
;** number of bombs to the level data.		**
;*************************************************

MakeLevel:
	move.l	#LvlData,a0	;<-- Let's get all the addresses 
	move.l	#WinData,a1	;<-- of all the arrays so we can clear 'em.
	move.l	#FlagData,a2		
	move.w	#9,d0			;<-- each array is 10 lines tall.

\ClearLineOfLevel
	clr.w		(a0)+			
	clr.w		(a1)+			;<-- CLEAR 'EM!
	clr.w		(a2)+
	dbf.w		d0,\ClearLineOfLevel	;<-- Loop 'till the cows come home.
	
	move.w	NumBombs,d7	;<-- Get the number of bombs this level is
						;	supposed to have.
	sub.w		#1,d7			;<-- Subtract 1 for the loop
	cmp.w		#-1,d7		;<-- If there are 0 bombs,then
						;	what the hell are we still doing here?!?
	bne		\AddABomb		;<-- May we add some bombs, please?
	rts					;<-- Let's go, go, go!

\AddABomb
	move.l	#LvlData,a0
	move.w	#16,d0		;<-- We need a number 1 - 16
	jsr		userlib::random	;<-- Get a random X
	move.w	d0,d1
	move.w	#10,d0		;<-- We need a number 1 - 10
	jsr		userlib::random	;<-- Get a random Y
	mulu.w	#2,d0			;<-- 2 bytes per line
	add.w		d0,a0
	move.w	(a0),d3		;<-- Get the data for the block we want.
	btst.l	d1,d3			;<-- Test to see if there's already a bomb there.
	bne		\AddABomb		;<-- If it's already a bomb, don't count it, skip
						;	back to the top of the loop.
	bset.l	d1,d3			;<-- It's free, so let's make it a bomb.
						;	We can't directly set the data, so we 
						;	move it into d3, set d3 accordingly, and
						;	THEN move it back into LvlData.
	move.w	d3,(a0)		;<-- Must move it back into (a0) to set the data
	dbf.w		d7,\AddABomb	;<-- Keep looping until we run out of bombs

	rts					;<-- GET OUT OF HERE! NOW! I MEAN IT!!!

;**************
;**** Lost ****
;*********************************
;** Displays that you have lost **
;** and starts the game again.	**
;*********************************

Lost:
	Square2	#32,#44,#96,#13			;<-- Create our word box.
	String	#35,#47,#15,#2,StrLost	;<-- Show the sad news in a box.
	jsr		Wait					;<-- Wait 'till they've read it.
	jmp 		Start					;<-- Then start the game all over again.

;*************
;**** Won ****
;*********************************
;** Displays that you have won, **
;** checks to see if you have a **
;** new high score, then starts **
;** the game again.			**
;*********************************

Won:
	Square2	#26,#44,#108,#13
	String	#29,#47,#17,#2,StrWon
	jsr		Wait
	jsr		CheckHighScore
	jmp		Start

;**************************
;**** Check High Score ****
;******************************************
;** Checks to see if you have a new high **
;** score. If so, it adds your name and  **
;** score to the list. Otherwise, it     **
;** returns to the calling code.         **
;******************************************

CheckHighScore:
	move.w	Seconds,d0
	cmp.w		High1,d0
	blt		NewHigh1
	cmp.w		High2,d0
	blt		NewHigh2
	cmp.w		High3,d0
	blt		NewHigh3
	cmp.w		High4,d0
	blt		NewHigh4
	cmp.w		High5,d0
	blt		NewHigh5
	rts

NewHigh1:
	Square2	#26,#60,#108,#13
	String	#29,#63,#17,#2,StrNewHigh
	jsr		Wait
	move.w	High4,High5
	move.w	High3,High4
	move.w	High2,High3
	move.w	High1,High2
	move.w	Seconds,High1
	move.l	#StrHigh1,a0
	move.l	#StrHigh2,a1
	move.l	#StrHigh3,a2
	move.l	#StrHigh4,a3
	move.l	#StrHigh5,a4
	move.w	#19,d0
\NewHigh1CopyLoop
	move.b	(a3),(a4)+
	move.b	(a2),(a3)+
	move.b	(a1),(a2)+
	move.b	(a0)+,(a1)+
	dbf.w		d0,\NewHigh1CopyLoop
	move.l	#StrHigh1,a6
	jsr		EnterName
	rts

NewHigh2:
	Square2	#26,#60,#108,#13
	String	#29,#63,#17,#2,StrNewHigh
	jsr		Wait
	move.w	High4,High5
	move.w	High3,High4
	move.w	High2,High3
	move.w	Seconds,High2
	move.l	#StrHigh2,a1
	move.l	#StrHigh3,a2
	move.l	#StrHigh4,a3
	move.l	#StrHigh5,a4
	move.w	#19,d0
\NewHigh2CopyLoop
	move.b	(a3),(a4)+
	move.b	(a2),(a3)+
	move.b	(a1)+,(a2)+
	dbf.w		d0,\NewHigh2CopyLoop
	move.l	#StrHigh2,a6
	jsr		EnterName
	rts

NewHigh3:
	Square2	#26,#60,#108,#13
	String	#29,#63,#17,#2,StrNewHigh
	jsr		Wait
	move.w	High4,High5
	move.w	High3,High4
	move.w	Seconds,High3
	move.l	#StrHigh3,a2
	move.l	#StrHigh4,a3
	move.l	#StrHigh5,a4
	move.w	#19,d0
\NewHigh3CopyLoop
	move.b	(a3),(a4)+
	move.b	(a2)+,(a3)+
	dbf.w		d0,\NewHigh3CopyLoop
	move.l	#StrHigh3,a6
	jsr		EnterName
	rts

NewHigh4:
	Square2	#26,#60,#108,#13
	String	#29,#63,#17,#2,StrNewHigh
	jsr		Wait
	move.w	High4,High5
	move.w	Seconds,High4
	move.l	#StrHigh4,a3
	move.l	#StrHigh5,a4
	move.w	#19,d0
\NewHigh4CopyLoop
	move.b	(a3)+,(a4)+
	dbf.w		d0,\NewHigh4CopyLoop
	move.l	#StrHigh4,a6
	jsr		EnterName
	rts

NewHigh5:
	Square2	#26,#60,#108,#13
	String	#29,#63,#17,#2,StrNewHigh
	jsr		Wait
	move.w	Seconds,High5
	move.l	#StrHigh5,a6
	jsr		EnterName
	rts

EnterName:
	move.l	a6,a1
	move.w	#0,NameOffset
	jsr		ClearScreen
	Square2	#17,#1,#126,#14
	StringA	#20,#4,#20,#2,a6
	Square2	#8,#16,#144,#77
	String	#11,#19,#23,#2,StrNHelp1
	String	#11,#28,#23,#2,StrNHelp2
	String	#11,#37,#23,#2,StrNHelp3
	String	#11,#46,#23,#2,StrNHelp4
	String	#11,#55,#23,#2,StrNHelp5
	String	#11,#64,#23,#2,StrNHelp6
	String	#11,#73,#23,#2,StrNHelp7
	String	#11,#82,#23,#2,StrNHelp8
	move.b	(a1),Char1
	jsr		PutChar1

\ENKeyB
	tst.w		(doorsos::kb_vars+$1c)
	beq		\ENKeyB
	move.w	(doorsos::kb_vars+$1e),d0
	clr.w		(doorsos::kb_vars+$1c)
	cmp.w		#KEY_UP,d0
	beq		\ENUp
	cmp.w		#KEY_DOWN,d0
	beq		\ENDown
	cmp.w		#KEY_LEFT,d0
	beq		\ENLeft
	cmp.w		#KEY_RIGHT,d0
	beq		\ENRight
	cmp.w		#13,d0
	beq		\ENEnter
	cmp.w		#120,d0
	beq		\ENSpace
	cmp.w		#121,d0
	beq		\ENUpperCase
	cmp.w		#122,d0
	beq		\ENLowerCase
	cmp.w		#116,d0
	beq		\ENNumbers
	bra		\ENKeyB

\ENUp
	move.b	(a1),d0
	add.w		#1,d0
	move.b	d0,(a1)
	move.b	d0,Char1
	StringA	#20,#4,#20,#2,a6
	jsr		PutChar1
	bra		\ENKeyB
	
\ENDown
	move.b	(a1),d0
	sub.w		#1,d0
	move.b	d0,(a1)
	move.b	d0,Char1
	StringA	#20,#4,#20,#2,a6
	jsr		PutChar1
	bra		\ENKeyB

\ENLeft
	tst.w		NameOffset
	beq		\ENLeftEnd
	suba.w	#1,a1
	sub.w		#1,NameOffset
	move.b	(a1),Char1	
	StringA	#20,#4,#20,#2,a6
	jsr		PutChar1
\ENLeftEnd
	bra		\ENKeyB

\ENRight
	cmpi.w	#19,NameOffset
	beq		\ENRightEnd
	adda.w	#1,a1
	add.w		#1,NameOffset
	move.b	(a1),Char1	
	StringA	#20,#4,#20,#2,a6
	jsr		PutChar1
\ENRightEnd
	bra		\ENKeyB

\ENUpperCase
	move.b	#65,(a1)
	move.b	#65,Char1
	StringA	#20,#4,#20,#2,a6
	jsr		PutChar1
	bra		\ENKeyB

\ENLowerCase
	move.b	#97,(a1)
	move.b	#97,Char1
	StringA	#20,#4,#20,#2,a6
	jsr		PutChar1
	bra		\ENKeyB

\ENNumbers
	move.b	#48,(a1)
	move.b	#48,Char1
	StringA	#20,#4,#20,#2,a6
	jsr		PutChar1
	bra		\ENKeyB

\ENSpace
	move.b	#32,(a1)
	move.b	#32,Char1
	StringA	#20,#4,#20,#2,a6
	jsr		PutChar1
	bra		\ENKeyB

\ENEnter
	rts

